#
#	Copyright 2021 Boris Daniel [X93]
#
#	PyLinuxTools is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	PyLinuxTools is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#	---------------------------------------------------------------------
#
#	Why PyLinuxTools?
#	PyLinuxTools its an all in one, it haves tools for control a lot of
#	things, like the backlight, leds, disks, batterys, procs and more!
#
#	And PyLinuxTools are Open Source and Free Software, following the
#	concept of Richard Stallman. Everybody can help, edit it redistribute
#	but maintaining the credits of the maintaners.
#

import os
import pylinuxtools.utils as utils
import pylinuxtools.errors as errors

def _if_backlight_exists(controler):
	"""
	Args:
		controler: str
		The controler to be wrapped
	"""
	if utils.direxists("/sys/class/backlight/{0}/".format(controler)):
		return True
	else:
		raise errors.HardwareError("Requested backlight controller does not exists")

def get_backlight_controller():
	"""
	Gets the backlight controllers
	"""
	return os.listdir("/sys/class/backlight/")

def get_backlight(controller=None):
	"""
	Get the actual backlight value

	Args:
		controller: str
		The controller to be wrapped
	"""
	if controller==None:
		controller=get_backlight_controller()[0]
	_if_backlight_exists(controller)
	return int(utils.get_sysfile("class/backlight/{0}/brightness".format(controller)))

def get_max_backlight(controller=None):
	"""
	Gets the max supported backlight

	Args:
		controller: str
		The controller to be wrapped
	"""
	if controller==None:
		controler=get_backlight_controller()[0]
	_if_backlight_exists(controller)
	return int(utils.get_sysfile("class/backlight/{0}/max_brightness".format(controller)))

def set_backlight(value: int, controller=None):
	"""
	Sets the backlight

	Args:
		value: int
		The value to be setted

		controller: str
		The controler to be wrapped
	"""
	if controller==None:
		controller=get_backlight_controller()[0]
	_if_backlight_exists(controller)
	if get_max_backlight(controller)>=value-1:
		file=open("/sys/class/backlight/{0}/brightness".format(controller), "w")
		file.write(str(value))
		file.close()
	else:
		raise errors.HardwareError("You haved reached the max supported backlight")
	return get_backlight()
