#
#	Copyright 2021 Boris Daniel [X93]
#
#	PyLinuxTools is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	PyLinuxTools is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#	---------------------------------------------------------------------
#
#	Why PyLinuxTools?
#	PyLinuxTools its an all in one, it haves tools for control a lot of
#	things, like the backlight, leds, disks, batterys, procs and more!
#
#	And PyLinuxTools are Open Source and Free Software, following the
#	concept of Richard Stallman. Everybody can help, edit it redistribute
#	but maintaining the credits of the maintaners.
#

import os
import pylinuxtools.utils as utils
import pylinuxtools.errors as errors

def _if_cpu_exists(cpu: str):
	"""
	Gets if an cpu exists

	Args:
		cpu: str
		The cpu to be wrapped
	"""
	if cpu in get_avaible_cpus():
		return True
	else:
		raise errors.HardwareError("CPU {0} does not exist".format(cpu))

def get_avaible_cpus():
	"""
	Gets the avaible cpu's on this PC
	"""
	n="0123456789"
	kill=False
	out=[]
	for i in os.listdir("/sys/devices/system/cpu/"):
		for a in i.replace("cpu", ""):
			if a in n:
				pass
			else:
				kill=True
				break
		if kill:
			kill=False
			continue
		else:
			out.append(i)
	out.sort()
	return out

def get_cpu_frequency_info(cpu: str):
	"""
	Gets the actual cpu frequency values

	Args:
		cpu: str
		The cpu to be wrapped
	"""
	_if_cpu_exists(cpu)
	return dict(
		current_frequency=int(utils.get_sysfile("devices/system/cpu/{0}/cpufreq/cpuinfo_cur_freq".format(cpu))),
		maximium_frequency=int(utils.get_sysfile("devices/system/cpu/{0}/cpufreq/cpuinfo_max_freq".format(cpu))),
		minimium_frequency=int(utils.get_sysfile("devices/system/cpu/{0}/cpufreq/cpuinfo_min_freq".format(cpu))),
	)

def get_cpu_scaling_frequency_info(cpu: str):
	"""
	Gets the actual cpu scaling frequency values

	Args:
		cpu: str
		The cpu to be wrapped
	"""
	_if_cpu_exists(cpu)
	return dict(
		current_frequency=int(utils.get_sysfile("devices/system/cpu/{0}/cpufreq/scaling_cur_freq".format(cpu))),
		maximium_frequency=int(utils.get_sysfile("devices/system/cpu/{0}/cpufreq/scaling_max_freq".format(cpu))),
		minimium_frequency=int(utils.get_sysfile("devices/system/cpu/{0}/cpufreq/scaling_min_freq".format(cpu))),
	)
