#
#	Copyright 2021 Boris Daniel [X93]
#
#	PyLinuxTools is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	PyLinuxTools is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#	---------------------------------------------------------------------
#
#	Why PyLinuxTools?
#	PyLinuxTools its an all in one, it haves tools for control a lot of
#	things, like the backlight, leds, disks, batterys, procs and more!
#
#	And PyLinuxTools are Open Source and Free Software, following the
#	concept of Richard Stallman. Everybody can help, edit it redistribute
#	but maintaining the credits of the maintaners.
#
#	Module:
#	Using the /dev/disk and /sys/block, we can obtain some information
#	about an disk, and partition's

import os
import subprocess
import pylinuxtools.utils as utils
import pylinuxtools.errors as errors

def _wrap_disk_partitions_dir(disk: str, dir: str="/sys/block/{0}/"):
	"""
	Wraps an disk partition dir, like /usb/ what on this
	directory exists an partitions name like this:
	{disk}1 and returns the list of partitions, this is
	useful when you charge an complete disk on a folder
	like /mnt and you want to unmount it recursively
	and more easyest

	Args:
		disk: str
		The disk raw name

		dir: str
		The directory to scan for partitions
	"""
	out=[]
	for i in os.listdir(dir.format(disk)):
		if disk in i:
			out.append(i)
	return out

def _if_disk_exists(disk: str):
	"""
	Scans if an battery are avaible

	Args:
		disk: str
		The disk raw name
	"""
	if disk in get_disks():
		return True
	else:
		raise errors.HardwareError("The requested disk "+disk+" dont exists!, [Normal in some cases]")

def _if_partition_exists(disk: str, partition: str):
	"""
	Matchs if an partition exists

	Args:
		disk: str
		The disk raw name

		partition: str
		The partition raw name
	"""
	if utils.direxists("/sys/block/{0}/{1}/".format(disk, partition)):
		return True
	else:
		raise errors.SoftwareError("The requested partition dont exists!, [Normal in some cases]")

def get_disks():
	"""
	Returns the connected disks
	"""
	_out=os.listdir("/sys/block")
	if len(_out)==0:
		raise errors.HardwareError("/sys/block/ dir not contains any disks (Not normal)")
	return _out

def get_removable_disks():
	"""
	Using the self funcion _get_disk_removable matchs all the
	removable disks
	"""
	_out=os.listdir("/sys/block/")
	for i in _out:
		if get_disk_removable(i)==False:
			_out.remove(i)
		else:
			pass
	return _out

def get_disk_removable(disk: str):
	"""
	Match's if an disk are removable

	Args:
		disk: str
		The disk raw name
	"""
	_if_disk_exists(disk)
	return int(utils.get_sysfile("block/{0}/removable".format(disk)))==1


class Disk:
	def __init__(self, disk: str):
		_if_disk_exists(disk)
		self.disk=disk

	def get_size(self, size_format: int=1):
		"""
		Return's the size of the requested disk
		"""
		return utils._return_formated(int(utils.get_sysfile("block/{0}/size".format(self.disk))), size_format)

	def get_partition_size(self, partition: int=1, size_format: int=1):
		"""
		Return's the size of the requested disk partition
		"""
		if type(partition)==int:
			partition=self.disk+str(partition)
			_if_partition_exists(self.disk, partition)
		else:
			_if_partition_exists(self.disk, partition)
		return utils._return_formated(int(open("/sys/block/{0}/{1}/size".format(self.disk, partition)).read()), size_format)

	def get_vendorinfo(self):
		"""
		Get's the disk vendor and model
		"""
		return dict(
			vendor=utils.get_sysfile("block/{0}/device/vendor".format(self.disk)),
			model=utils.get_sysfile("block/{0}/device/model".format(self.disk)),
		)

	def list_partitions(self):
		"""
		Return's now avaible disk partitions
		"""
		return _wrap_disk_partitions_dir(self.disk)

	def get_partition(self, partition: int=1):
		"""
		Return's an requested partition.

		Args:
			partition: int
			The partition ID
		"""
		if type(partition)==int:
			partition=self.disk+str(partition)
			_if_partition_exists(self.disk, partition)
		else:
			_if_partition_exists(self.disk, partition)
		return Partition(self, partition)

class Partition:
	def __init__(self, disk: Disk, partition: str):
		"""
		Initializes an Partition class

		Args:
			disk: Disk
			The main disk class

			partition: str
			The partition name
		"""
		_if_partition_exists(disk, partition)
		self.disk=disk
		self.partition=partition

	def get_label(self):
		"""
		Return's the partition label

		Note: Raises SoftwareError if the partition dont
		have's an label
		"""
		for partition in os.listdir("/dev/disk/by-label"):
			partition_rawname=os.readlink("/dev/disk/by-label/"+partition)
			partition_rawname=partition_rawname.split("/")[len(partition_rawname.split("/"))-1]
			if self.partition==partition_rawname:
				return partition
		raise errors.SoftwareError("The partition dont have's an label")

	def get_size(self, size_format: int=1):
		"""
		Returns the partition size

		Args:
			size_format: int
			Go to the utils._return_formated documentation
		"""
		return self.disk._get_partition_size(self.partition, size_format)
	def mount(self, mount_point: str="/mnt"):
		"""
		Mounts an partition with the mount shell command

		Args:
			mount_point: str
			The place on the disk should be mounted
		"""
		out=subprocess.run("mount -v /dev/"+self.partition+" "+mount_point, encoding="UTF-8", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=False)
		o={}
		for i in out.stderr.split('\n'):
			if i=='':
				print(i)
				continue
			if ':' in i:
				o.update({i.split(':' ,maxsplit=1)[0]: i.split(':', maxsplit=1)[1]})
			else:
				o.update({i:''})
		return out.returncode, out.stdout, o
	def unmount(self, mount_point: str="/mnt"):
		"""
		Unmount the partition with the umount shell command

		Args:
			mount_point: str
			The place on the disk what's mounted
		"""
		out=subprocess.run("umount -v "+mount_point, encoding="UTF-8", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=False)
		o={}
		for i in out.stderr.split('\n'):
			if i=='':
				print(i)
				continue
			if ':' in i:
				o.update({i.split(':' ,maxsplit=1)[0]: i.split(':', maxsplit=1)[1]})
			else:
				o.update({i:''})
		return out.returncode, out.stdout, o
