#
#	Copyright 2021 Boris Daniel [X93]
#
#	PyLinuxTools is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	PyLinuxTools is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#	---------------------------------------------------------------------
#
#	Why PyLinuxTools?
#	PyLinuxTools its an all in one, it haves tools for control a lot of
#	things, like the backlight, leds, disks, batterys, procs and more!
#
#	And PyLinuxTools are Open Source and Free Software, following the
#	concept of Richard Stallman. Everybody can help, edit it redistribute
#	but maintaining the credits of the maintaners.
#

import pylinuxtools.utils as utils

def get_bios_info():
	"""
	Get the self bios info
	"""
	return dict(
		bios_date=utils._get_kernel_file("/sys/class/dmi/id/bios_date"),
		bios_release=utils._get_kernel_file("/sys/class/dmi/id/bios_release"),
		bios_vendor=utils._get_kernel_file("/sys/class/dmi/id/bios_vendor"),
		bios_version=utils._get_kernel_file("/sys/class/dmi/id/bios_version"),
	)

def get_board_info():
	"""
	Gets the self board info
	"""
	return dict(
		board_asset_tag=utils._get_kernel_file("/sys/class/dmi/id/board_asset_tag"),
		board_name=utils._get_kernel_file("/sys/class/dmi/id/board_name"),
		board_serial=utils._get_kernel_file("/sys/class/dmi/id/board_serial"),
		board_vendor=utils._get_kernel_file("/sys/class/dmi/id/board_vendor"),
	)

def get_chassis_info():
	"""
	Gets the self chassis info
	"""
	return dict(
		chassis_asset_tag=utils._get_kernel_file("/sys/class/dmi/id/chassis_asset_tag"),
		chassis_type=utils._get_kernel_file("/sys/class/dmi/id/chassis_type"),
		chassis_serial=utils._get_kernel_file("/sys/class/dmi/id/chassis_serial"),
		chassis_vendor=utils._get_kernel_file("/sys/class/dmi/id/chassis_vendor"),
		chassis_version=utils._get_kernel_file("/sys/class/dmi/id/chassis_version"),
	)

def get_product_info():
	"""
	Gets the self product info
	"""
	return dict(
		product_family=utils._get_kernel_file("/sys/class/dmi/id/product_family"),
		product_name=utils._get_kernel_file("/sys/class/dmi/id/product_name"),
		product_serial=utils._get_kernel_file("/sys/class/dmi/id/product_serial"),
		product_sku=utils._get_kernel_file("/sys/class/dmi/id/product_sku"),
		product_uuid=utils._get_kernel_file("/sys/class/dmi/id/product_uuid"),
		product_version=utils._get_kernel_file("/sys/class/dmi/id/product_version"),
	)

def get_ec_firmware_release():
	"""
	Returns the self ec firmware release
	"""
	return utils._get_kernel_file("/sys/class/dmi/id/ec_firmware_release")

def get_sys_vendor():
	"""
	Returns the system sys vendor using the
	dmi dir
	"""
	return utils._get_kernel_file("/sys/class/dmi/id/sys_vendor")
