#
#	Copyright 2021 Boris Daniel [X93]
#
#	PyLinuxTools is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	PyLinuxTools is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#	---------------------------------------------------------------------
#
#	Why PyLinuxTools?
#	PyLinuxTools its an all in one, it haves tools for control a lot of
#	things, like the backlight, leds, disks, batterys, procs and more!
#
#	And PyLinuxTools are Open Source and Free Software, following the
#	concept of Richard Stallman. Everybody can help, edit it redistribute
#	but maintaining the credits of the maintaners.
#

import os
import pylinuxtools.utils as utils
import pylinuxtools.errors as errors

def _if_graphic_exists(graphic: str):
	if graphic in get_graphics():
		return True
	else:
		raise errors.HardwareError("Graphic {0} does not exists")

def _if_graphic_sbpp(graphic: str):
	if utils.fileexists("/sys/class/graphics/{0}/bits_per_pixel".format(graphic)):
		return True
	else:
		raise errors.HardwareError("Graphic {0} does not support bits per pixel".format(graphic))

def _if_graphic_ss(graphic: str):
	if utils.fileexists("/sys/class/graphics/{0}/virtual_size".format(graphic)):
		return True
	else:
		raise errors.HardwareError("Graphic {0} does not support virtual sizing".format(graphic))

def get_graphics():
	return os.listdir("/sys/class/graphics/")

def get_graphic_bits_per_pixel(graphic: str):
	_if_graphic_exists(graphic)
	_if_graphic_sbpp(graphic)
	return int(utils._get_kernel_file("/sys/class/graphics/{0}/bits_per_pixel".format(graphic)))

def get_graphic_virtual_size(graphic: str):
	_if_graphic_exists(graphic)
	_if_graphic_ss(graphic)
	__size__=utils._get_kernel_file("/sys/class/graphics/{0}/virtual_size".format(graphic)).split(",", maxsplit=1)
	return dict(
		width=__size__[0],
		height=__size__[1]
	)

def get_graphic_fullname(graphic: str):
	_if_graphic_exists(graphic)
	return utils._get_kernel_file("/sys/class/graphics/{0}/name".format(graphic))
