#
#	Copyright 2021 Boris Daniel [X93]
#
#	PyLinuxTools is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	PyLinuxTools is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#	---------------------------------------------------------------------
#
#	Why PyLinuxTools?
#	PyLinuxTools its an all in one, it haves tools for control a lot of
#	things, like the backlight, leds, disks, batterys, procs and more!
#
#	And PyLinuxTools are Open Source and Free Software, following the
#	concept of Richard Stallman. Everybody can help, edit it redistribute
#	but maintaining the credits of the maintaners.
#

import os
import time
import pylinuxtools.utils as utils
import pylinuxtools.errors as errors

pathlib=utils.pathlib

# Pathlib walk method
# created for use it on
# this project

def iter_dir(dir):
	"""
	Wraps an dirs

	Args:
		dir: str
		The dir to be wrapped
	"""
	d=[]
	f=[]
	path=pathlib.Path(dir)
	for i in path.iterdir():
		if i.is_dir():
			d.append(i.name)
		else:
			f.append(i.name)
	return dir, d, f

def wrap_iter(i: list):
	"""
	Wraps the response of walk method

	Args:
		i: list
		The list with the files and dirs
	"""
	o=[i]
	dir=i[0]
	for i in i[1]:
		o+=walk(dir+"/"+i)
	return o

def walk(dir: str):
	"""
	Walk's on a dir and search subdir's for walk in them again

	Args:
		dir: str
		The dir to be walked

	Note: This are an connection for wrap_iter and iter_dir
	"""
	it=iter_dir(dir)
	out=[it]
	if it[1]==[]:
		pass
	else:
		for i in it[1]:
			o=iter_dir(dir+'/'+i)
			out+=wrap_iter(o)
	return out

# Self module

def _if_kernel_exists(kernel: str):
	"""
	Returns True if the kernel exists else raises an error

	Args:
		kernel: str
		The kernel to be wrapped
	"""
	if kernel in get_kernels():
		return True
	else:
		raise errors.SoftwareError("Kernel dont exists [Normal in some cases]")

def get_kernel_mods(kernel: str):
	"""
	Returns a list with all the mods of the kernel

	Args:
		kernel: str
		The kernel to be wrapped
	"""
	_if_kernel_exists(kernel)
	out=[]
	for i in walk("/lib/modules/{0}".format(kernel)):
		if i[2]==[]:
			continue
		for c in i[2]:
			if "." not in c:
				continue
			if c.split(".", maxsplit=1)[1] in ["ko.zst", "ko.xz"]:
				out.append(c.rstrip(".ko.xz").rstrip(".ko.zst"))
	return out

def get_kernels():
	"""
	Returns a list with the avaible kernels
	"""
	return os.listdir("/lib/modules/")

def modprobe(kernel: str, mod: str):
	"""
	Using get_kernel_mods function match if the kernel
	haves support for an specific module, like the functionally
	of modprobe shell command.

	Args:
		kernel: str
		The kernel to be wrapped

		mod: str
		The mod what will be searched
	"""
	_if_kernel_exists(kernel)
	for i in get_kernel_mods(kernel):
		if mod==i:
			return True
	raise errors.SoftwareError("Module dont exists on the kernel "+kernel)

def get_hostname():
	return open("/proc/sys/kernel/hostname").read().strip("\r\n")