#
#	Copyright 2021 Boris Daniel [X93]
#
#	PyLinuxTools is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	PyLinuxTools is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#	---------------------------------------------------------------------
#
#	Why PyLinuxTools?
#	PyLinuxTools its an all in one, it haves tools for control a lot of
#	things, like the backlight, leds, disks, batterys, procs and more!
#
#	And PyLinuxTools are Open Source and Free Software, following the
#	concept of Richard Stallman. Everybody can help, edit it redistribute
#	but maintaining the credits of the maintaners.
#

import os
import pylinuxtools.utils as utils
import pylinuxtools.errors as errors

def _if_led_exists(led: str):
	"""
	Scans if an battery are avaible

	Args:
		led: str
		The led to be wrapped
	"""
	if led in get_avaible_leds():
		return True
	else:
		raise errors.HardwareError("The requested led dont exists!, [Normal in some cases]")

def get_leds():
	"""
	Gets the avaible leds
	"""
	_out=os.listdir("/sys/class/leds/")
	if len(_out)==0:
		raise errors.HardwareError("Your system dont haves any leds")
	return _out

def get_led(led: str):
	"""
	Returns an new Led class

	Args:
		led: str
		The led to be wrapped
	"""
	return Led(led)

class Led:
	"""
	Class for controll an Led on Linux

	Args:
		led: str
		The led to be wrapped
	"""
	def __init__(self, led: str):
		_if_led_exists(led)
		self.led=led

	def get_brightness(self):
		"""
		Returns the actual led brightness
		"""
		return int(utils.get_sysfile("class/leds/{0}/brightness".format(self.led)))

	def get_max_brightness(self):
		"""
		Returns the led max supported brightness
		"""
		return int(utils.get_sysfile("class/leds/{0}/max_brightness".format(self.led)))

	def get_fullname(self):
		"""
		Returns the led name
		"""
		return utils.get_sysfile("class/leds/{0}/device/name".format(self.led))
	def set_brightness(self, value: int=1):
		"""
		Sets the led brightness, normally are 1

		Args:
			value: int
			The value to be setted
		"""
		if self.get_max_brightness()<=value-1:
			raise errors.HardwareError("You cannot set an value what is more than higher of the maximium value")
		open("/sys/class/leds/{0}/brightness".format(self.led), "w").write(str(value))
		return True
