#include<sys/ioctl.h>
#include<fcntl.h>
#include<stdio.h>
#include<linux/cdrom.h>

// Sucess output states
#define CDROM_OPEN 1
#define CDROM_CLOSED 0
#define CDROM_SUCESS 2
#define CDROM_ALREADY 3

// ERROR's
#define CDROM_UNDEFINED -1
#define CDROM_IOCTL_ERROR -2
#define CDROM_INVALID -3

// Check's the state of an CDROM
// Return's CDROM_CLOSED, CDROM_OPEN
// and CDROM_IOCTL_ERROR
int check_state(int cdrom){
	if (cdrom==CDROM_IOCTL_ERROR){
		return CDROM_INVALID;
	}

	int cdrom_stat;
	if ((cdrom_stat = ioctl(cdrom, CDROM_DRIVE_STATUS, 0)) < 0){
		return CDROM_UNDEFINED;
	}
	else{
		if (cdrom_stat==CDS_TRAY_OPEN){
			return CDROM_OPEN;
		}
		else{
			return CDROM_CLOSED;
		}
	}
}

// Eject's the cdrom
int eject_cdrom(int cdrom){
	if (cdrom==CDROM_IOCTL_ERROR){
		return CDROM_INVALID;
	}

	int stat=check_state(cdrom);

	if ((stat==0) || (stat==-1)){
		if (ioctl(cdrom, CDROMEJECT, 0) < 0){
			return CDROM_IOCTL_ERROR;
		}
		else{
			return CDROM_SUCESS;
		}
	}
	else{
		return CDROM_ALREADY;
	}
}

// Close's the cdrom
int close_cdrom(int cdrom){
	if (cdrom==CDROM_IOCTL_ERROR){
		return CDROM_INVALID;
	}

	int stat=check_state(cdrom);

	if ((stat==1) || (stat==-1)){
		if (ioctl(cdrom, CDROMCLOSETRAY, 0) < 0){
			return CDROM_IOCTL_ERROR;
		}
		else{
			return CDROM_SUCESS;
		}
	}
	else{
		return CDROM_ALREADY;
	}
}

// Get's the CDROM
int get_cdrom(const char *cdrom){
	int cdrom_f;

	if ((cdrom_f=open(cdrom, O_RDONLY | O_NONBLOCK)) < 0){
		return CDROM_IOCTL_ERROR;
	}
	else{
		return cdrom_f;
	}
}

int toogle_cdrom(const char *cdrom){
	int cdrom_file=get_cdrom(cdrom);

	if (cdrom_file==CDROM_IOCTL_ERROR){
		return CDROM_INVALID;
	}
	int stat=check_state(cdrom_file);

	if (stat==1){
		int close_stat=close_cdrom(cdrom_file);
		if (close_stat==CDROM_IOCTL_ERROR){
			return CDROM_IOCTL_ERROR;
		}
		else if (close_stat==CDROM_INVALID){
			return CDROM_INVALID;
		}

		return CDROM_CLOSED;
	}
	else if (stat==0){
		int eject_stat=eject_cdrom(cdrom_file);
		if (eject_stat==CDROM_IOCTL_ERROR){
			return CDROM_IOCTL_ERROR;
		}
		else if (eject_stat==CDROM_INVALID){
			return CDROM_INVALID;
		}

		return CDROM_OPEN;
	}
	else{
		return CDROM_INVALID;
	}
}

int main(){
	// Testing
	int stat=toogle_cdrom("/dev/sr0");
	const char *response;

	switch(stat){
		case -2:{
			response="IOCTL Error";
			break;
		}
		case -1:{
			response="Invalid CDROM";
			break;
		}
		case 0:{
			response="Closed";
			break;
		}
		case 1:{
			response="Opened";
			break;
		}
		case 2:{
			response="Sucess";
			break;
		}
	}

	printf("Status: %s\r\n", response);
}
