#
#	Copyright 2021 Boris Daniel [X93]
#
#	PyLinuxTools is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	PyLinuxTools is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#	---------------------------------------------------------------------
#
#	Why PyLinuxTools?
#	PyLinuxTools its an all in one, it haves tools for control a lot of
#	things, like the backlight, leds, disks, batterys, procs and more!
#
#	And PyLinuxTools are Open Source and Free Software, following the
#	concept of Richard Stallman. Everybody can help, edit it redistribute
#	but maintaining the credits of the maintaners.
#

import os
import pylinuxtools.utils as utils
import pylinuxtools.errors as errors

def _if_power_exists(by_id: str="BAT0"):
	"""
	Scans if an battery are avaible

	Args:
		by_id: str
		The power supply name
	"""
	if by_id in get_power_supplys():
		return True
	else:
		raise errors.HardwareError("The requested battery dont exists!, [Normal in some cases]")

def get_power_status(by_id="BAT0"):
	"""
	Gets the battery status

	Args:
		by_id: str
		The power supply name
	"""
	_if_power_exists(by_id=by_id)

	return utils.get_sysfile("class/power_supply/{0}/status".format(by_id))

def get_power_energy(by_id="BAT0"):
	"""
	Gets the battery actual energy, energy_full, and the
	energy_full_design

	Args:
		by_id: str
		The power supply name
	"""
	_if_power_exists(by_id=by_id)

	return dict(
		energy_now=int(utils.get_sysfile("class/power_supply/{0}/energy_now".format(by_id))),
		energy_full=int(utils.get_sysfile("class/power_supply/{0}/energy_full".format(by_id))),
		energy_full_design=int(utils.get_sysfile("class/power_supply/{0}/energy_full_design".format(by_id))),
	)

def get_power_percent(by_id="BAT0"):
	"""
	Gets the battery percent using the actual energy
	and calculating the mathematical percent

	Args:
		by_id: str
		The power supply name
	"""
	battery=get_power_energy(by_id=by_id)
	result=battery["energy_now"]*100/battery["energy_full"]
	_result=battery["energy_now"]*100/battery["energy_full_design"]
	return dict(
			actual_percent=result,
			design_percent=_result,
		)

def get_power_supplys():
	"""
	Gets the battery actual adapters
	"""
	if utils.direxists('/sys/class/power_supply/'):
		return os.listdir("/sys/class/power_supply/")
	else:
		raise errors.HardwareError("In this PC dont exists any battery controler or power supply, [Not normal]")

def get_power_deviceinfo(by_id="BAT0"):
	"""
	Returns the power supply device info

	Args:
		by_id: str
		The power supply name
	"""
	_if_power_exists(by_id)

	return dict(
			model_name=utils.get_sysfile("class/power_supply/{0}/model_name".format(by_id)),
			manufacturer=utils.get_sysfile("class/power_supply/{0}/manufacturer".format(by_id)),
			serial_number=utils.get_sysfile("class/power_supply/{0}/serial_number".format(by_id)),
			technology=utils.get_sysfile("class/power_supply/{0}/technology".format(by_id)),
		)

# Examples
# print(get_power_status())
# print(get_power_energy())
# print(get_power_percent())
# print(get_power_supplys())
# print(get_power_deviceinfo())
