import os

def get_processors():
	processors={}
	for proc in open("/proc/cpuinfo").read().split("\n\n"):
		props={}
		if proc=="":
			continue

		for property in proc.split("\n"):
			property=property.split(":")
			property[0]=property[0].strip("\t").replace(" ", "_").lower()
			if property[0]=="":
				continue
			if property[1] in ["yes", "no"]:
				props.update({property[0]: True if property[1]=="yes" else False})
				continue
			if "." in property[1]:
				props.update({property[0]: float(property[1])})
				continue
			try:
				props.update({property[0]: int(property[1])})
			except:
				props.update({property[0]: str(property[1])})
		processors.update({props["processor"]: Processor(**props)})
	return processors

class Processor:
	"""
	A container for get_processors
	"""
	def __init__(self,
			processor: int=0,
			vendor_id: str="",
			cpu_family: str="",
			model: str="",
			model_name: str="",
			stepping: str="",
			microcode: str="",
			cpu_mhz: float=0.0,
			cache_size: str="",
			physical_id: int=0,
			siblings: int=0,
			core_id: int=0,
			cpu_cores: int=0,
			apicid: int=0,
			initial_apicid: int=0,
			fpu: bool=False,
			fpu_exception: bool=False,
			cpuid_level: int=0,
			wp: bool=False,
			flags: str="",
			bugs: str="",
			bogomips: float=0.0,
			tlb_size: str="",
			clflush_size: int=0,
			cache_alignment: int=0,
			address_sizes: str="",
			power_management: str=""
		):
		self.processor=processor
		self.vendor_id=vendor_id
		self.cpu_family=cpu_family
		self.model=model
		self.model_name=model_name
		self.stepping=stepping
		self.microcode=microcode
		self.cpu_mhz=cpu_mhz
		self.cache_size=cache_size
		self.physical_id=physical_id
		self.siblings=siblings
		self.core_id=core_id
		self.cpu_cores=cpu_cores
		self.apicid=apicid
		self.initial_apicid=initial_apicid
		self.fpu=fpu
		self.fpu_exception=fpu_exception
		self.cpuid_level=cpuid_level
		self.wp=wp
		self.flags=flags
		self.bugs=bugs
		self.bogomips=bogomips
		self.tlb_size=tlb_size
		self.clflush_size=clflush_size
		self.cache_alignment=cache_alignment
		self.address_sizes=address_sizes
		self.power_management=power_management

	def todict(self):
		return dict(
			processor=self.processor,
			vendor_id=self.vendor_id,
			cpu_family=self.cpu_family,
			model=self.model,
			model_name=self.model_name,
			stepping=self.stepping,
			microcode=self.microcode,
			cpu_mhz=self.cpu_mhz,
			cache_size=self.cache_size,
			physical_id=self.physical_id,
			siblings=self.siblings,
			core_id=self.core_id,
			cpu_cores=self.cpu_cores,
			apicid=self.apicid,
			initial_apicid=self.initial_apicid,
			fpu=self.fpu,
			fpu_exception=self.fpu_exception,
			cpuid_level=self.cpuid_level,
			wp=self.wp,
			flags=self.flags,
			bugs=self.bugs,
			bogomips=self.bogomips,
			tlb_size=self.tlb_size,
			clflush_size=self.clflush_size,
			cache_alignment=self.cache_alignment,
			address_sizes=self.address_sizes,
			power_management=self.power_management,
		)

