#
#	Copyright 2021 Boris Daniel [X93]
#
#	PyLinuxTools is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	PyLinuxTools is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#	---------------------------------------------------------------------
#
#	Why PyLinuxTools?
#	PyLinuxTools its an all in one, it haves tools for control a lot of
#	things, like the backlight, leds, disks, batterys, procs and more!
#
#	And PyLinuxTools are Open Source and Free Software, following the
#	concept of Richard Stallman. Everybody can help, edit it redistribute
#	but maintaining the credits of the maintaners.
#

import os
import subprocess
import pylinuxtools.utils as utils
import pylinuxtools.errors as errors

def _if_proc_exists(proc_id: int):
	"""
	Matchs if the requested proc exists

	Args:
		proc_id: int
		The proc to be wrapped
	"""
	if utils.direxists("/proc/{0}/".format(str(int(proc_id)))):
		return True
	else:
		raise errors.SoftwareError("The requested proc does not exists")

def get_proc_info(proc_id: int):
	"""
	Gets the info of a proc like the Name and other things

	Args:
		proc_id: int
		The proc to be wrapped
	"""
	_if_proc_exists(proc_id)
	_out=dict()
	for i in open("/proc/{0}/status".format(str(int(proc_id)))).read().split("\n"):
		if i=="":
			continue
		i=i.replace("\t", "")
		_out.update({
			i.split(":", maxsplit=1)[0]
				:
			i.split(":", maxsplit=1)[1]}
			if ":" in i else {
			i: ""
			})
	return _out

def actual_procs(_type=1):
	"""
	Returns the actual running proc list
	"""
	if _type==1:
		_out=list()
		for i in os.listdir("/proc/"):
			try:
				_out.append(int(i))
			except:
				pass
		return _out
	elif _type==0:
		_out=dict()
		for i in os.listdir("/proc/"):
			try:
				_=int(i)
				_info=_get_proc_info(_)
				if _info["Name"] in list(_out.keys()):
					ch=_out[_info["Name"]] if type(_out[_info["Name"]])==list else [_out[_info["Name"]]]
					_out[_info["Name"]]=[*ch, _]
				else:
					_out.update({_info["Name"]: _})
			except Exception as e:
				pass
		return _out
	else:
		raise ValueError("_type arg only can be 1 or 0")

def get_meminfo():
	props={}
	for property in open("/proc/meminfo"):
		property=property.split(":")
		property[0]=property[0].strip("\n").strip(" ")
		property[1]=property[1].strip("\n").strip(" ")
		props.update({property[0]: property[1]})
	return props

def kill_proc(proc_id: int):
	"""
	Kills a proc with the kill shell command

	Args:
		proc_id: int
		The proc to be killed
	"""
	_if_proc_exists(int(proc_id))
	proc=subprocess.run("kill "+str(int(proc_id)), check=False ,shell=True, stderr=subprocess.PIPE, encoding='UTF-8')
	if proc.returncode==1:
		return True
	else:
		raise errors.SoftwareError("Kill command failed")
