#
#	Copyright 2021 Boris Daniel [X93]
#
#	PyLinuxTools is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	PyLinuxTools is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	You should have received a copy of the GNU General Public License
#	GNU General Public License for more details.
#
#	along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#	---------------------------------------------------------------------
#
#	Why PyLinuxTools?
#	PyLinuxTools its an all in one, it haves tools for control a lot of
#	things, like the backlight, leds, disks, batterys, procs and more!
#
#	And PyLinuxTools are Open Source and Free Software, following the
#	concept of Richard Stallman. Everybody can help, edit it redistribute
#	but maintaining the credits of the maintaners.
#

import os
import pylinuxtools.utils as utils
import pylinuxtools.errors as errors

def _if_video_exists(video: str):
    if video in get_video_adapters():
        return True
    else:
        raise errors.HardwareError("We can't found the video adapter %s" % video)

def get_video_adapters():
    """
    Return's the avaible video adapter's
    """
    devices=os.listdir("/sys/class/video4linux/")
    if len(devices)==0:
        raise errors.HardwareError("You don't have's camera's connected to this dispositive")
    else:
        return devices

def get_video_is_authorized(id: str):
    """
    Return's if a video driver are authorized, but normally this are unnecesary
    because if the device are unauthorized, you cannot access it.

    Args:
        id: str
        The id of the device
    """
    _if_video_exists(id)
    return utils.get_sysfile("class/video4linux/%s/device/authorized" % id)

def get_video_modelname(id: str):
    """
    Return's the video4linux device model

    Args:
        id: str
        The id of the device
    """
    _if_video_exists(id)
    try:
        return utils.get_sysfile("class/video4linux/%s/device/media0/model" % id)
    except:
        return "unsoported"

def get_video_name(id: str):
    """
    Return's the video4linux device name

    Args:
        id: str
        The id of the device
    """
    _if_video_exists(id)
    return utils.get_sysfile("class/video4linux/%s/name" % id)

def get_video_interface(id: str):
    """
    Return's the video4linux device interface

    Args:
        id: str
        The id of the device
    """
    _if_video_exists(id)
    try:
        return utils.get_sysfile("class/video4linux/%s/device/interface" % id)
    except:
        return "unsoported"

class Video4Linux:
    authorized: bool
    model_name: str
    interface: str
    name: str
    id: str

    def __init__(self, id: str="video0"):
        """
        Wrap's an video4linux device

        Args:
            id: str
            The id of the device
        """

        _if_video_exists(id)
        # In some cases, if the camera are not authorized, you can't access it
        # so self.authorized are True in every case.
        self.authorized=True if get_video_is_authorized(id)=="1" else False

        # Some camera's, don't support the media0 directory so self.model_name
        # be unsoported, but normally the name are the same as the model_name
        self.model_name=get_video_modelname(id)

        self.id=id
        self.name=get_video_name(id)

        # The interface variable maybe contains the first model_name, but if the
        # model_name are unsoported, this feature, maybe be unsoported
        self.interface=get_video_interface(id)

    def todict(self):
        # Convert's the declared variable's of the self.__init__ function
        # to an dict
        return dict(
            id=self.id,
            name=self.name,
            interface=self.interface,
            authorized=self.authorized,
            model_name=self.model_name,
        )

    def unauthorize(self):
        """
        This function disables the video input for re enable
        it you need to restart
        """

        # When you do this you are killing literally your camera, until the next
        # reboot
        open("/sys/class/video4linux/%s/device/authorized" % self.id, "w").write("0")
        return False
